
####################################################
##### R Programming Example 1 for Chapter 4 ########
#####         Comparing two conditions      ########        
####################################################

#We read gene expression data from an external file, 
#and assume that the data was already normalized by an appropriate method.
#So, we jsut take a log2-transformation.
#The T-cell immune response (Affymetrix MG-U74Av2) data was described in Chapter 4. 
#The data contains 3 replicates for each of 3 conditions: 
#(1) Naive, (2) 48h activated, (3) CD8+ T-cell clone D4

MGU = read.csv("Tcell.csv") #12,488 probe sets * 9 samples 
dim(MGU)
head(MGU)
narrays = ncol(MGU)
ngenes  = nrow(MGU)
log.MGU=log2(MGU[,2:ncol(MGU)])  #log2-transformation
head(log.MGU)

x1 = log.MGU[,1:3]
x2 = log.MGU[,4:6]
x12 = log.MGU[,1:6] # 

#To run the following example, we need to install the following R libraries:
# Biobase, multtest, limma, samr, LPE
library(Biobase)
library(multtest)
library(limma)
library(samr)
library(LPE)


#Set a seed
set.seed(1234)


##########################
##### Fold Change ########
##########################

mean.x1 = apply(2^x1,1,mean,na.rm=T)
mean.x2 = apply(2^x2,1,mean,na.rm=T)
logFC = log2(mean.x1/mean.x2)


##########################
### Two-Sample T-Test ####
###      (Welch)      ####
##########################

Ttest = function(y, group){
   fit = t.test(y~group, var.equal=FALSE) #Welch's t-test
   out = c(fit$statistic, fit$p.value) 
   return(out)
}

Tstats = data.frame(matrix(NA, ngenes, 5))
colnames(Tstats) = c("t-stat","raw.p.value.t", "Bonf.adj.p.value.t", "BH.adj.p.value.t","BY.adj.p.value.t")

group = c(rep(0, 3), rep(1,3))
Tstats[,1:2] = t(apply(cbind(x1,x2), 1, Ttest, group=group))

#Applying multiple testing procedures
res = mt.rawp2adjp(Tstats[,2], c("Bonferroni", "BH","BY"))
adjp = res$adjp[order(res$index),]
Tstats[,2:5] = adjp
head(Tstats)


##########################
# Wilcoxon Rank-sum Test #
##########################

Wtest = function(y, group){
   fit = wilcox.test(y~group)
   out = c(fit$statistic, fit$p.value) 
   return(out)
}

Wstats = data.frame(matrix(NA, ngenes, 5))
colnames(Wstats) = c("W-stat","raw.p.value.W", "Bonf.adj.p.value.W", "BH.adj.p.value.W", "BY.adj.p.value.W")
group = c(rep(0, 3), rep(1,3))
Wstats[,1:2] = t(apply(cbind(x1,x2), 1, Wtest, group=group))

#Applying multiple testing procedures
res = mt.rawp2adjp(Wstats[,2], c("Bonferroni","BH","BY"))
adjp = res$adjp[order(res$index),]
Wstats[,2:5] = adjp
head(Wstats)


############################
####### LIMMA ##############
############################

LMstats = data.frame(matrix(NA, ngenes, 4))
colnames(LMstats) = c("limma-stat","raw.p.value.limma", "BH.adj.p.value.limma","BY.adj.p.value.limma")

design <- cbind(Grp1=1,Grp2vs1=c(0,0,0,1,1,1))
fit <- lmFit(cbind(x2,x1),design)
fit <- eBayes(fit)

tmp <- topTable(fit, coef=2, number=nrow(LMstats), adjust.method="BH")[,3:5]
tmp <- tmp[sort.list(as.numeric(rownames(tmp))),]
LMstats[,1:3] <- tmp
tmp <- topTable(fit, coef=2, number=nrow(LMstats), adjust.method="BY")[,3:5]
tmp <- tmp[sort.list(as.numeric(rownames(tmp))),]
LMstats[,4] <- tmp[,3]
head(LMstats)


##########################
####### SAM ##############
##########################
	
group = c(rep(1, 3), rep(2,3))
data=list(x=cbind(x2,x1), y=group, geneid=as.character(1:ngenes), 
     genenames=paste("g",as.character(1:nrow(x12)),sep=""), logged2=TRUE)
samr.obj = samr(data,  resp.type="Two class unpaired", nperms=100)
delta=-1

samr.plot(samr.obj,delta)

delta.table = samr.compute.delta.table(samr.obj)
siggenes.table = samr.compute.siggenes.table(samr.obj,delta, data, delta.table)
out = rbind(siggenes.table$genes.up, siggenes.table$genes.lo) 
i = sort.list(as.numeric(out[,3]))
out = out[i,]
Dstats = matrix(as.numeric(out[,c(4,8)]), nrow(out),2)
colnames(Dstats) = c("SAM-stat", "q.value.SAM(%)")
head(Dstats)

##########################
####### LPE ##############
##########################

var.1 = baseOlig.error(x1, q=0.01)
var.2 = baseOlig.error(x2, q=0.01)
lpe.result = lpe(x1,x2,var.1,var.2,probe.set.name=MGU[,1])
lpe.fdr = fdr.adjust(lpe.result, adjp="resamp", ALL=TRUE)
k = order(abs(lpe.result$z.stats), decreasing = TRUE)
Lstats = lpe.fdr[order((1:length(k))[k]),c(2,1)]
Lstats[,1] = lpe.result$z.stats
colnames(Lstats) = c("LPE-stat", "FDR.LPE")
head(Lstats)


###########################
######### Summary #########
###########################

out = cbind(MGU[,1], logFC, Tstats, Wstats, LMstats, Dstats, Lstats)
colnames(out) = c("GeneID", "LogFC",
                  "t-stat","raw.p.value.t", "Bonf.adj.p.value.t", "BH.adj.p.value.t", "BY.adj.p.value.t",
                  "W-stat","raw.p.value.W", "Bonf.adj.p.value.W", "BH.adj.p.value.W", "BY.adj.p.value.W",
                  "limma-stat","raw.p.value.limma", "BH.adj.p.value.limma", "BY.adj.p.value.limma",
                  "SAM-stat", "q.value.SAM(%)", 
                  "LPE-stat", "FDR.LPE")
write.table(out, file="Results.Example1.csv", sep=",", row.names=FALSE)



###########################
#########   END   #########
###########################
