
####################################################
##### R Programming Example 2 for Chapter 4 ########
#####      Comparing multiple conditions    ########
####################################################

#We read gene expression data from an external file, 
#and assume that the data was already normalized by an appropriate method.
#So, we jsut take a log2-transformation.
#The T-cell immune response (Affymetrix MG-U74Av2) data was described in Chapter 4. 
#The data contains 3 replicates for each of 3 conditions: 
#(1) Naive, (2) 48h activated, (3) CD8+ T-cell clone D4

MGU = read.csv("Tcell.csv") #12,488 probe sets * 9 samples 
dim(MGU)
head(MGU)
narrays = ncol(MGU)
ngenes  = nrow(MGU)
log.MGU=log2(MGU[,2:ncol(MGU)])  #log2-transformation
head(log.MGU)

x1 = log.MGU[,1:3]
x2 = log.MGU[,4:6]
x3 = log.MGU[,7:9]
x123 = log.MGU[,1:9] # x123  ʾ ߰Ͽϴ.

#To run the following example, we need to install the following R libraries:
# Biobase, multtest, samr, HEM
library(Biobase)
library(multtest)
library(samr)
library(HEM)

#Set a seed
set.seed(1234)


##########################
###   ANOVA F-Test    ####
##########################

Ftest = function(y, group){
   fit = anova(lm(y~factor(group)))
   out = c(fit$`F value`[1], fit$`Pr(>F)`[1]) 
   return(out)
}

Fstats = data.frame(matrix(NA, ngenes, 5))
colnames(Fstats) = c("F-stat","raw.p.value.F", 
                     "Bonf.adj.p.value.F", "BH.adj.p.value.F", "BY.adj.p.value.t")

group = c(rep(0,3),rep(1,3),rep(2,3))
Fstats[,1:2] = t(apply(cbind(x1,x2,x3), 1, Ftest, group=factor(group)))

#Applying multiple testing procedures
res = mt.rawp2adjp(Fstats[,2], c("Bonferroni", "BH","BY"))
adjp = res$adjp[order(res$index),]
Fstats[,2:5] = adjp
head(Fstats)


############################
##  Krukskal-Wallis Test ###
############################

KWtest = function(y, group){
   fit = kruskal.test(y~factor(group))
   out = c(fit$statistic, fit$p.value) 
   return(out)
}

KWstats = data.frame(matrix(NA, ngenes, 5))
colnames(KWstats) = c("KW-stat","raw.p.value.KW", 
                      "Bonf.adj.p.value.KW", "BH.adj.p.value.KW", "BY.adj.p.value.KW")

group = c(rep(0,3),rep(1,3),rep(2,3))
KWstats[,1:2] = t(apply(cbind(x1,x2,x3), 1, KWtest, group=factor(group)))


#Applying multiple testing procedures
res = mt.rawp2adjp(KWstats[,2], c("Bonferroni", "BH","BY"))
adjp = res$adjp[order(res$index),]
KWstats[,2:5] = adjp
head(KWstats)


##########################
########## SAM ###########
##########################

group = c(rep(1, 3), rep(2,3),rep(3,3))
data=list(x=cbind(x1,x2,x3), y=group, geneid=as.character(1:nrow(x123)), 
     genenames=paste("g",as.character(1:nrow(x123)),sep=""), logged2=TRUE)

samr.obj = samr(data,  resp.type="Multiclass", nperms=100)
delta=-0.1

samr.plot(samr.obj,delta)

delta.table = samr.compute.delta.table(samr.obj)
siggenes.table = samr.compute.siggenes.table(samr.obj,delta, data, delta.table)

out = rbind(siggenes.table$genes.up, siggenes.table$genes.lo) 
i = sort.list(as.numeric(out[,3]))
out = out[i,]
Dstats = matrix(as.numeric(out[,c(4,10)]), nrow(out),2)
colnames(Dstats) = c("SAM-stat", "q.value.SAM(%)")
head(Dstats)

##########################
########## HEM ###########
##########################

cond <- c(1,1,1,2,2,2,3,3,3) #conditions
rep  <- c(1,2,3,1,2,3,1,2,3) #replicates
design <- data.frame(cond,rep)

MGU.eb  <- hem.eb.prior(cbind(x1,x2,x3), n.layer=1, design=design, B=25, method.var.t="neb")
MGU.hem <- hem(cbind(x1,x2,x3), n.layer=1, design=design, burn.ins=1000, n.samples=3000,  method.var.t="neb", var.t=MGU.eb$var.t)
Hstats <- hem.fdr(cbind(x1,x2,x3), n.layer=1, design=design, n.iter=1, hem.out=MGU.hem, eb.out=MGU.eb)$fdr
colnames(Hstats) = c("HEM-stat", "FDR.HEM")
head(Hstats)


###########################
######### Summary #########
###########################

out = cbind(MGU[,1], Fstats, KWstats, Dstats, Hstats)
colnames(out) = c("GeneID", 
                  "F-stat","raw.p.value.F", "Bonf.adj.p.value.F", "BH.adj.p.value.F", "BY.adj.p.value.F",
                  "KW-stat","raw.p.value.KW", "Bonf.adj.p.value.KW", "BH.adj.p.value.KW","BY.adj.p.value.KW",
                  "SAM-stat", "q.value.SAM(%)", 
                  "HEM-stat", "FDR.HEM")
write.table(out, file="Results.Example2.csv", sep=",", row.names=FALSE)


###########################
#########   END   #########
###########################

