
####################################################
##### R Programming Example 3 for Chapter 4 ########
#####        Comparing paired samples       ########        
####################################################

#The platelet (LC-MS/MS) data was described in Chapter 4. 
#The data contains 3 replicates for two paired samples: Treated vs Untreated

data(platelet)
x <- log2(platelet) #any transformation/normalization
dim(x)  #62 peptides * 3 replicates of 2 paired samples 
head(x)


#To run the following example, we need to install the following R libraries:
# Biobase, multtest, limma, samr, PLPE
library(Biobase)
library(multtest)
library(limma)
library(samr)
library(PLPE)


#Set a seed
set.seed(1234)


##########################
##### Fold Change ########
##########################

xx = 2^x
nr = ncol(xx)/2
FC = xx[,1]/xx[,2]
for(i in 2:nr) FC = FC + xx[,(2*i-1)]/xx[,(2*i)]
logFC = log2(FC/nr)


##########################
###    Paired T-Test  ####
##########################

Ttest = function(y, group){
   k = group[1]
   y1 = y[(group==k)]; y2 = y[!(group==k)]
   fit = t.test(y1, y2, paired=TRUE)
   out = c(fit$statistic, fit$p.value) 
   return(out)
}

Tstats = data.frame(matrix(NA, nrow(x), 5))
colnames(Tstats) = c("paired.t-stat","raw.p.value.t", "Bonf.adj.p.value.t", "BH.adj.p.value.t","BY.adj.p.value.t")

group = c(1,2,1,2,1,2)
Tstats[,1:2] = t(apply(x, 1, Ttest, group=group))

#Applying multiple testing procedures
res = mt.rawp2adjp(Tstats[,2], c("Bonferroni", "BH","BY"))
adjp = res$adjp[order(res$index),]
Tstats[,2:5] = adjp
head(Tstats)


#############################
# Wilcoxon Signed-rank Test #
#############################

Wtest = function(y, group){
   k = group[1]
   y1 = y[(group==k)]; y2 = y[!(group==k)]
   fit = wilcox.test(y1, y2, paired=TRUE)
   out = c(fit$statistic, fit$p.value) 
   return(out)
}

Wstats = data.frame(matrix(NA, nrow(x), 5))
colnames(Wstats) = c("W-stat","raw.p.value.W", "Bonf.adj.p.value.W", "BH.adj.p.value.W", "BY.adj.p.value.W")

group = c(1,2,1,2,1,2)
Wstats[,1:2] = t(apply(x, 1, Wtest, group=group))

#Applying multiple testing procedures
res = mt.rawp2adjp(Wstats[,2], c("Bonferroni","BH","BY"))
adjp = res$adjp[order(res$index),]
Wstats[,2:5] = adjp
head(Wstats)


############################
####### LIMMA ##############
############################

LMstats = data.frame(matrix(NA, nrow(x), 4))
colnames(LMstats) = c("LIMMA-stat","raw.p.value.LIMMA", "BH.adj.p.value.LIMMA","BY.adj.p.value.LIMMA")

Repl <- factor(c(1,1,2,2,3,3))
Treat <- factor(c(2,1,2,1,2,1))
design <- model.matrix(~Repl+Treat)

fit <- lmFit(x, design)
fit <- eBayes(fit)
tmp <- topTable(fit, coef="Treat2", number=nrow(LMstats), adjust.method="BH")[,4:6]
tmp <- tmp[sort.list(as.numeric(rownames(tmp))),]
LMstats[,1:3] <- tmp
tmp <- topTable(fit, coef="Treat2",number=nrow(LMstats), adjust.method="BY")[,4:6]
tmp <- tmp[sort.list(as.numeric(rownames(tmp))),]
LMstats[,4] <- tmp[,3]
head(LMstats)


##########################
####### SAM ##############
##########################
	
group = c(1, -1, 2, -2, 3, -3)
data=list(x=x, y=group, geneid=as.character(1:nrow(x)), 
     genenames=paste("g",as.character(1:nrow(x)),sep=""), logged2=TRUE)
samr.obj = samr(data,  resp.type="Two class paired", nperms=100)
delta=-10

samr.plot(samr.obj, delta)

delta.table = samr.compute.delta.table(samr.obj)
siggenes.table = samr.compute.siggenes.table(samr.obj,delta, data, delta.table)
out = rbind(siggenes.table$genes.up, siggenes.table$genes.lo) 
i = sort.list(as.numeric(out[,3]))
out = out[i,]
Dstats = matrix(as.numeric(out[,c(4,8)]), nrow(out),2)
colnames(Dstats) = c("SAM-stat", "q.value.SAM(%)")
head(Dstats)


##########################
####### PLPE ##############
##########################

cond <- c(1, 2, 1, 2, 1, 2)
pair <- c(1, 1, 2, 2, 3, 3)
design <- cbind(cond, pair)

out <- lpe.paired(x, design, q=0.1, probe.ID=1:nrow(x),  data.type="ms")
out.fdr <- lpe.paired.fdr(x,obj=out)
Lstats = out.fdr$FDR[,1:2]
colnames(Lstats) = c("PLPE-stat", "FDR.PLPE")
head(Lstats)


###########################
######### Summary #########
###########################

out = cbind(1:nrow(x), logFC, Tstats, Wstats, LMstats, Dstats, Lstats)
colnames(out) = c("PeptideID", "LogFC",
                  "paired.t-stat","raw.p.value.t", "Bonf.adj.p.value.t", "BH.adj.p.value.t", "BY.adj.p.value.t",
                  "W-stat","raw.p.value.W", "Bonf.adj.p.value.W", "BH.adj.p.value.W","BY.adj.p.value.W",
                  "limma-stat","raw.p.value.limma", "BH.adj.p.value.limma", "BY.adj.p.value.limma",
                  "SAM-stat", "q.value.SAM(%)", 
                  "PLPE-stat", "FDR.PLPE")
write.table(out, file="Results.Example3.csv", sep=",", row.names=FALSE)


###########################
#########   END   #########
###########################
